// Copyright 2023 Michael Amann and contributors
// SPDX-License-Identifier: AGPL-3.0-only

package packagelist

import (
	"fmt"
	"net/http"
	"strings"

	"git.kle.li/tools/go-import-redirector/pkg/configuration/file"
)

type PackageList struct {
	packages map[string]*Package
	host     string
}

const (
	templateErrorMsg  = "Failed to parse template."
	templateErrorCode = 500
)

func NewPackageList(host string, modules []*file.Module) *PackageList {
	pl := &PackageList{
		packages: map[string]*Package{},
		host:     host,
	}
	for _, m := range modules {
		pl.addNewPackage(m)
	}

	return pl
}

type Package struct {
	Path, Import, Source, Repo, Home, Title string
}

func newPackage(m *file.Module, host string) *Package {
	p := &Package{
		Path: m.Path,
		Repo: m.Repo,
	}

	p.genValues(m, host)

	return p
}

func (pl *PackageList) addNewPackage(m *file.Module) {
	p := newPackage(m, pl.host)
	pl.packages[p.Path] = p
}

func (p *Package) genValues(m *file.Module, host string) {
	p.Import = host + m.Path + " " + m.VcsType + " " + m.Repo

	if m.Title == "" {
		p.Title = "Package " + host + p.Path
	} else {
		p.Title = m.Title
	}

	switch strings.ToLower(m.SourceType) {
	case "forgejo":
		m.DirectoryURL = m.Repo + "/src/branch/" + m.Branch + "{/dir}"
		m.FileURL = m.DirectoryURL + "/{file}#L{line}"
	case "github":
		m.DirectoryURL = m.Repo + "/tree/" + m.Branch + "{/dir}"
		m.FileURL = m.DirectoryURL + "/{file}#L{line}"
	case "gitlab":
		m.DirectoryURL = m.Repo + "/-/tree/" + m.Branch + "{/dir}"
		m.FileURL = m.Repo + "/-/blob/" + m.Branch + "{/dir}" + "/{file}#L{line}"
	case "manual":
		p.Home = m.HomeURL
		p.buildSourceString(m, host)
		return
	default:
		return
	}

	// handle generic auto settings...
	if m.HomeURL == "" {
		m.HomeURL = m.Repo
	}
	p.Home = m.HomeURL
	p.buildSourceString(m, host)
}

func (p *Package) buildSourceString(m *file.Module, host string) {
	if isNonValue(m.HomeURL) && isNonValue(m.DirectoryURL) && isNonValue(m.FileURL) {
		return
	}

	if m.HomeURL == "" {
		m.HomeURL = "_"
	}

	if m.DirectoryURL == "" {
		m.DirectoryURL = "_"
	}

	if m.FileURL == "" {
		m.FileURL = "_"
	}

	p.Source = host + m.Path + " " + m.HomeURL + " " + m.DirectoryURL + " " + m.FileURL
}

func isNonValue(s string) bool {
	s = strings.TrimSpace(s)
	if s == "" || s == "_" {
		return true
	}
	return false
}

func (p *Package) String() string {
	return fmt.Sprintf("\nPath: %v\nTitle: %v\nRepo: %v\nHome: %v\nImport: %v\nSource: %v\n", p.Path, p.Title, p.Repo, p.Home, p.Import, p.Source)
}

func (pl *PackageList) GetPackages() map[string]*Package {
	return pl.packages
}

func (pl *PackageList) GetHost() string {
	return pl.host
}

func (pl *PackageList) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	if r.Host != pl.host {
		http.NotFound(w, r)
		return
	}

	if r.URL.Path == "/" {
		if pkg, ok := pl.GetMostSpecificPackage(""); ok {
			err := indexTmpl.Execute(w, map[string]interface{}{
				"host": pl.host,
				"pkgs": pl.packages,
				"pkg":  pkg,
			})
			if err != nil {
				http.Error(w, templateErrorMsg, templateErrorCode)
			}
			return
		}
		err := indexTmpl.Execute(w, map[string]interface{}{
			"host": pl.host,
			"pkgs": pl.packages,
		})
		if err != nil {
			http.Error(w, templateErrorMsg, templateErrorCode)
		}
		return
	}
	fmt.Println("Path is " + r.URL.Path)
	if pkg, ok := pl.GetMostSpecificPackage(r.URL.Path); ok {
		err := pkgTmpl.Execute(w, map[string]interface{}{
			"pkg": pkg,
		})
		if err != nil {
			http.Error(w, templateErrorMsg, templateErrorCode)
		}
		return
	}
	http.NotFound(w, r)
}

func (pl *PackageList) GetMostSpecificPackage(path string) (*Package, bool) {
	if p, ok := pl.packages[path]; ok {
		return p, ok
	}

	for segment := path; segment != ""; segment = segment[:strings.LastIndex(segment, "/")] {
		if p, ok := pl.packages[segment]; ok {
			return p, ok
		}
	}

	return nil, false
}
